/*
  Lithium 0.1

  Copyright (C) 2001 Teti Paolo.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  (LICENSE file) along with this program; if not, write to
  the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/


package lithium2;

/**
 *
 * <br>The Divide&Conquer Skeleton.</br>
 *
 *
 * @author Paolo Teti
 * @version 0.1
 */

public class DivideConquer extends JSkeletons
{

    JSkeletons dcWorker = null; // The DC Worker
    Condition condition = null; // The base-condition
    Divide divideRule = null; // The divide Rule
    Recomp recompRule = null; // The Recompisition Rule


    /**
     *
     * Costruct a Divide&Conquer Skeleton
     *
     * @param dcWorker The worker to be executed
     * @param c The base-condition
     * @param drule The rule used to divide the task in subtask
     * @param rrule The rule used to recopose the subtask in task
     */
    public DivideConquer(JSkeletons dcWorker, Condition c,
			 Divide drule, Recomp rrule)
    {
	this.dcWorker = dcWorker;
	divideRule = drule;
	condition = c;
	recompRule = rrule;
    }


    // Packet structure
    // --------------------------------------------------
    // | CONDITION | DCWORKER | DIVIDERULE | RECOMPRULE |
    //---------------------------------------------------
    // 0           1          2            3
    Object[] getSkeletonInfo()
    {
	Object[] info  = new Object[4];

	info[0] = condition;
	info[1] = dcWorker;
	info[2] = divideRule;
	info[3] = recompRule;
	return (info);
    }


    /**
     * Execute the skeleton in sequenzial mode.
     * (i.e like a sequential skeleton)
     *
     * @param task The task to compute.
     * @return The result of the computation.
     */
    public Object run(Object task)
    {
	Object[] tmpTask;
	Object[] tmpReturnTask;

	if (condition.apply(task))
	    // Resolve directly the atomic task
	    return dcWorker.run(task);
	else
	    {
		// Apply the Recomposition and Divide Rule
		// on a generic task. (recurs.)
		tmpTask = divideRule.run(task);
		tmpReturnTask = new Object[tmpTask.length];

		for (int i=0;i<tmpTask.length;i++)
		    tmpReturnTask[i] = (this.run(tmpTask[i]));

		return (recompRule.run(tmpReturnTask));
	    }
    }
} // DivideConquer
