/*
  Lithium 0.1

  Copyright (C) 2001 Teti Paolo.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  (LICENSE file) along with this program; if not, write to
  the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/


package lithium2;
import java.util.Vector;


/**
 * <b> The Map skeleton </b>
 *
 *
 * @author Paolo Teti
 * @version 0.1
 */
public class Map extends JSkeletons
{
    public static final int ROW = 0;
    public static final int COLUMN = 1;
    public static final int ELEMENTS = 2;
    public static final int BROADCAST = 3;
    JSkeletons mapWorker = null;
    Vector ruleList = null;
    int dimX = 0;
    int dimY = 0;

    /**
     *The Map skeleton costructor
     *
     *@param sk The skeleton to be executed
     */
    public Map(JSkeletons sk)
    {
	mapWorker = sk;
	ruleList = new Vector();
    }

    /**
     * set a the rule
     * @param rule a Rule
     * @exception BadRuleException if r is a bad rule
     */
    public void setParamRule(int rule) throws BadRuleException
    {
	if (rule<ROW && rule>BROADCAST) throw new
	    BadRuleException(LithiumExc.BADRULE);
	ruleList.add(new Integer(rule));
    }

    /**
     * set the out rule
     * @param or a Rule
     * @param dimX X-dimension of the matrix
     * @param dimY Y-dimension of the matrix
     * @exception BadRuleException if r is a bad rule
     */
    public void setOutRule(int or,int dimX, int dimY) throws BadRuleException
    {
	if (or<ROW && or>BROADCAST) throw new
	    BadRuleException(LithiumExc.BADRULE);
	ruleList.add(new Integer(or));
	ruleList.add(new Integer(dimX));
	ruleList.add(new Integer(dimY));
	this.dimX = dimX;
	this.dimY = dimY;
    }


    Object[] getSkeletonInfo()
    {
	// Packet structure
	// ----------------------------------------------
	// | RULES | OUT RULE | DIMX | DIMY | MAPWORKER |
	// ----------------------------------------------
	// 0       K         K+1    K+2     K+3

	Object[] tmpSkel = new Object[ruleList.size()+1];
	for (int j=0;j<ruleList.size();j++)
	    tmpSkel[j] = ruleList.elementAt(j);

	tmpSkel[ruleList.size()] = mapWorker;
	return (tmpSkel);
    }



    public Object run(Object task)
    {
	MapUtils eval = new MapUtils(ruleList.toArray());

	Object [][] o = eval.applyRule((Object[])task);

	// Sequential Emulation...
	for (int i=0;i<eval.getVP()[0];i++)
	    for (int j=0;j<eval.getVP()[1];j++)
		eval.recomp(mapWorker.run(((Vector)o[i][j]).
					  toArray()),i,j);

	return (Object)eval.getResult();
    }

} // Map

