/*
  Lithium 0.1

  Copyright (C) 2001 Teti Paolo.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public
  License (LICENSE file) along with this program; if not, write
  to the Free Software Foundation, Inc., 59 Temple Place, Suite
  330, Boston, MA 02111-1307 USA
*/

package lithium2;

import java.util.Vector;

class MapUtils
{
    public static final int ROW = 0;
    public static final int COLUMN = 1;
    public static final int ELEMENTS = 2;
    public static final int BROADCAST = 3;
    private Integer[] ruleList= null;
    private int numOfRule = LithiumConst.UNDEF;
    private int outRule = LithiumConst.UNDEF;
    private Object[][] realRetValue = null;
    private int dimX = LithiumConst.UNDEF;
    private int dimY = LithiumConst.UNDEF;
    private int numOfMappingElement = 0;

    public MapUtils(Object[] ruleList)
    {
	numOfRule = ruleList.length;
	this.ruleList = new Integer[numOfRule];
	System.arraycopy(ruleList,0,this.ruleList,0,numOfRule);
	dimY = ((Integer)ruleList[numOfRule-1]).intValue();
	dimX = ((Integer)ruleList[numOfRule-2]).intValue();
	outRule = ((Integer)ruleList[numOfRule-3]).intValue();
	numOfRule-=3;
	realRetValue = new Object[dimX][dimY];
    }


    public Object[][] applyRule(Object[] task)
    {
	Object [][] pack = new Object [dimX][dimY];
	int ruleType = LithiumConst.UNDEF;

	Vector genericData = new Vector();

	for (int t=0;t<dimX;t++)
	    for (int k=0;k<dimY;k++)
		{
		    int positionX = 0;
		    int positionY = 0;
		    for (int i=0;i<numOfRule;i++)
			{
			    ruleType = ((Integer)ruleList[i]).intValue();
			    switch (ruleType)
				{

				case ROW:
				    {
					genericData.add(((Object[][])task[i])[positionX]);
					positionX++;
					break;
				    }

				case COLUMN:
				    {
					int n = ((Object[][])task[i])[0].length;
					Object[] tmp = new Object[n];

					for (int l=positionY;l<n;l++)
					    tmp[(l-positionY)] =
						((Object[][])task[i])[positionX][l];

					positionY++;
					genericData.add(tmp);
					break;
				    }

				case  ELEMENTS:
				    {
					genericData.add(((Object[][])task[i])
							[positionX][positionY]);
					positionX++;
					positionY++;
					break;
				    }

				case BROADCAST:
				    {
					genericData.add(task[i]);
					break;
				    }

				} // Switch

			}

		    pack[t][k] = genericData;
		}
	return (pack);
    }


    public int [] getVP()
    {
	int [] vp = new int[2];

	switch (outRule)
	    {

	    case ROW:
		{
		    vp[0] = dimX;
		    vp[1] = 1;
		    break;
		}
	    case COLUMN:
		{
		    vp[0] = 1;
		    vp[1] = dimY;
		    break;
		}

	    case ELEMENTS:
		{
		    vp[0] = dimX;
		    vp[1] = dimY;
		    break;
		}
	    case BROADCAST:
		{
		    vp[0] = 1;
		    vp[1] = 1;
		}
	    }
	return vp;
    }


    public boolean isOperationComplete()
    {
	return (numOfMappingElement==(dimX*dimY));
    }

    public Object [][] getResult()
    {
	return (realRetValue);
    }

    public void recomp(TaskItem[] t)
    {
	for (int i=0;i<t.length;i++)
	    recomp(t[i].dataTask,t[i].toXVP[t[i].lastVP],t[i].toYVP[t[i].lastVP]);
    }

    public void recomp(Object o, int x, int y)
    {
	realRetValue[x][y] = o;
	/*
	switch (outRule)
	    {

	    case ROW:
		{
		    realRetValue[x][y] = o
		    break;
		}

	    case COLUMN:
		{
		    realRetValue[x][y] = o
		    break;
		}

	    case ELEMENTS:
		{
		    realRetValue[x][y] = o;
		    break;
		}

	    }
	*/
	numOfMappingElement++;
    }

} // MapUtils

